package com.pj.tb_wallet_record.app;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONUtil;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.exception.HttpException;
import com.wechat.pay.java.core.exception.MalformedMessageException;
import com.wechat.pay.java.core.exception.ServiceException;
import com.wechat.pay.java.service.payments.app.AppService;
import com.wechat.pay.java.service.payments.app.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;

/** AppService使用示例 */
public class AppServiceExample {

  /** 商户号 */
  public static String merchantId = "1695540177";

  /** 商户API私钥路径 */
  public static String privateKeyPath = "E:\\libs\\wx\\apiclient_key.pem";

  /** 商户证书序列号 */
  public static String merchantSerialNumber = "5469DA5ED227341CF27A93AE1D31811D6965A139";

  /** 商户APIV3密钥 */
  public static String apiV3Key = "r73q9l7im6qtle33eua3l5ogp6lahss8";

  public static AppService service;

  public static void main(String[] args) {
    // 初始化商户配置
    Config config =
        new RSAAutoCertificateConfig.Builder()
            .merchantId(merchantId)
            // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
            .privateKeyFromPath(privateKeyPath)
            .merchantSerialNumber(merchantSerialNumber)
            .apiV3Key(apiV3Key)
            .build();

    // 初始化服务
    service = new AppService.Builder().config(config).build();
    // ... 调用接口
    try {
      PrepayResponse response=prepay();
      System.out.println("prepay response: " + JSONUtil.toJsonStr(response));
    } catch (HttpException e) { // 发送HTTP请求失败
      e.printStackTrace();
      // 调用e.getHttpRequest()获取请求打印日志或上报监控，更多方法见HttpException定义
    } catch (ServiceException e) { // 服务返回状态小于200或大于等于300，例如500
      e.printStackTrace();
      // 调用e.getResponseBody()获取返回体打印日志或上报监控，更多方法见ServiceException定义
    } catch (MalformedMessageException e) { // 服务返回成功，返回体类型不合法，或者解析返回体失败
      e.printStackTrace();
      // 调用e.getMessage()获取信息打印日志或上报监控，更多方法见MalformedMessageException定义
    }
  }

  /** 关闭订单 */
  public static void closeOrder() {

    CloseOrderRequest request = new CloseOrderRequest();
    // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
    // 调用接口
    service.closeOrder(request);
  }

  /** APP支付下单 */
  public static PrepayResponse prepay() {
    PrepayRequest request = new PrepayRequest();
    // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
    request.setAppid("wx0daaef365288a0db");
    request.setMchid(merchantId);
    request.setOutTradeNo(RandomUtil.randomString(16));
    request.setDescription("Image形象店-深圳腾大-QQ公仔");
    request.setNotifyUrl("http://hs-server.tbgjhc.top:8018/level-one-server/TbPeople/getList");
    Amount amount=new Amount();
    amount.setTotal(1);
    amount.setCurrency("CNY");
    request.setAmount(amount);
    // 调用接口
    return service.prepay(request);
  }

  /** 微信支付订单号查询订单 */
  public static Transaction queryOrderById() {

    QueryOrderByIdRequest request = new QueryOrderByIdRequest();
    // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
    // 调用接口
    return service.queryOrderById(request);
  }

  /** 商户订单号查询订单 */
  public static Transaction queryOrderByOutTradeNo() {

    QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
    // 调用request.setXxx(val)设置所需参数，具体参数可见Request定义
    // 调用接口
    return service.queryOrderByOutTradeNo(request);
  }
}
