package com.pj.tb_wallet_topdown;

import java.util.List;
import com.pj.tb_wallet_topdown.vo.WalletTopdownVo;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.pj.utils.sg.*;
import com.pj.project4sp.SP;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_wallet_topdown -- 服务点提现记录
 * @author plj 
 */
@RestController
@RequestMapping("/TbWalletTopdown/")
public class TbWalletTopdownController {

	/** 底层 Service 对象 */
	@Autowired
	TbWalletTopdownService tbWalletTopdownService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE_ADD)
	public AjaxJson add(TbWalletTopdown t){
		tbWalletTopdownService.add(t);
		t = tbWalletTopdownService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbWalletTopdownService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(TbWalletTopdown.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbWalletTopdown t){
		tbWalletTopdownService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE)
	public AjaxJson getById(Long id){
		TbWalletTopdown t = tbWalletTopdownService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<WalletTopdownVo> list = tbWalletTopdownService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	/** 改 - 删除状态(0=禁用,1=启用) */  
	@RequestMapping("updateDeleteStatus")
	@SaCheckPermission(TbWalletTopdown.PERMISSION_CODE_EDIT)
	public AjaxJson updateDeleteStatus(Long id, Integer value){
		int line = SP.publicMapper.updateColumnById(TbWalletTopdown.TABLE_NAME, "delete_status", value, id);
		return AjaxJson.getByLine(line);
	}


}
